// Set display key (localStorage)
function setDisplayKey () {
    var displayKey = prompt('Please Enter your Display Key');
    localStorage.setItem('_dk', displayKey);
    window.displayKey = displayKey;

    try {
        chrome.tabs.executeScript({
            "code": "var myScript = document.createElement('script');"
            + "myScript.textContent = 'localStorage.setItem(\\'_dk\\', \\'" + displayKey + "\\');';"
            + "document.head.appendChild(myScript);"
        });
    } catch(e) { console.log(e) }
}

// Set calendar key (localStorage)
function setCalendarKey () {
    var calKey = prompt('Please Enter your Calendar Key');
    localStorage.setItem('_calkey', calKey);

    try {
        chrome.tabs.executeScript({
            "code": "var myScript = document.createElement('script');"
            + "myScript.textContent = 'localStorage.setItem(\\'_calkey\\', \\'" + calKey + "\\');';"
            + "document.head.appendChild(myScript);"
        });
    } catch(e) { console.log(e) }

    getMeetings();
}

// Send a command to the child window
function sendCommand (command, tabId) {
    if (!command) command = prompt('Enter your command');

    try {
        chrome.tabs.executeScript(tabId, {
            "code": "var myScript = document.createElement('script');"
            + "myScript.textContent = 'localStorage.setItem(\\'_rcmd\\', \\'" + command + "\\');';"
            + "document.head.appendChild(myScript);"
        });
    } catch(e) { console.log(e) }
}

// Set up context menu items
if (location.protocol != "chrome:") {
    //chrome.contextMenus.create({
    //    id: 'set-display-key',
    //    title: "Set Display Key",
    //    onclick: setDisplayKey
    //});

    chrome.contextMenus.create({
        id: 'set-calendar-key',
        title: "Set Meeting Calendar Key",
        onclick: setCalendarKey
    });

    chrome.contextMenus.create({
        id: 'go-to-current-meeting',
        title: "Set Tab as TV Display",
        onclick: goToCurrentMeeting
    });

    chrome.contextMenus.create({
        id: 'capture',
        title: "Make Volume Adjustable",
        onclick: function () { adjustVolume(100) }
    });

    chrome.contextMenus.create({
        id: 'send-remote-command',
        title: "Send Command",
        onclick: function () { sendCommand() }
    });
}

function adjustVolume (volume) {
    chrome.tabs.query({ audible: true }, function (tabs) {
        tabs.forEach(function (tab) {
            if (tab && tab.id) {
                console.log('audiotab:', tab.id);

                var message = {
                    tabId: tab.id,
                    sliderValue: volume
                };

                if (Object.prototype.hasOwnProperty.call(audioStates, message.tabId)) {
                    updateVolume(message.tabId, message.sliderValue);
                    // background.updateBadge(message.tabId, message.sliderValue)
                } else {
                    chrome.tabCapture.capture({
                        audio: true,
                        video: false
                    }, (stream) => {
                        if (chrome.runtime.lastError) {
                            console.log(chrome.runtime.lastError);
                        } else {
                            connectTab(message.tabId, stream);
                            updateVolume(message.tabId, message.sliderValue);
                            // background.updateBadge(message.tabId, message.sliderValue)
                        }
                    });
                }
            }
        });
    });
}

function goToCurrentMeeting () {
    localStorage.setItem('_currentMeetingURL', 'no');

    chrome.tabs.query({ active: true, currentWindow: true }, function (tabs) {
        tabs.forEach(function (tab) {
            if (tab && tab.id) {
                chrome.tabs.update(tab.id, { url: 'https://teams.microsoft.com/' } );
                setTimeout(getMeetings, 1500);
            }
        });
    });

    adjustVolume(100);
    getMeetings();
}

function goToTVURL (url) {
    // alert('going to: ' + url);

    var _currentMeetingURL = localStorage.getItem('_currentMeetingURL');

    if (_currentMeetingURL == url) return;

    chrome.tabs.query({}, function (tabs) {

        tabs.forEach(function (tab) {

            if (tab && tab.id) {
                // Navigate to TV display
                if (tab.url.includes('quik.chat') || tab.url.includes('teams.microsoft')) {
                    localStorage.setItem('_currentMeetingURL', url);
                    chrome.tabs.update(tab.id, { url: url } );
                }
            }

        });
    });
}

function shuffle (a) {
    var j, x, i;
    for (i = a.length - 1; i > 0; i--) {
        j = Math.floor(Math.random() * (i + 1));
        x = a[i];
        a[i] = a[j];
        a[j] = x;
    }
    return a;
}

function trigraph () {
    var words = "find any new work part take get place made live where after back little only round year came show every good me give our under name very through just form sentence great think say help low line differ turn cause much mean before move right boy old too same tell does set three want air well also play small end put home read hand port large spell add even land here must big high such follow act why ask change went light kind off need house picture try us again animal point mother world near build self earth father head stand own page should country found answer school grow study still learn plant cover food sun four between state keep eye never last let thought city tree cross farm hard start might story saw far sea draw left late run dont while press close night real life few north open seem together next white children begin got walk example ease paper group always music those both mark often letter until mile river car feet care second book carry took science eat room friend began idea fish mountain stop once base hear horse cut sure watch color face wood main enough plain girl usual young ready above ever red list though feel talk bird soon body dog family direct pose leave song measure door product black short numeral class wind question happen complete ship area half rock order fire south problem piece told knew pass since top whole king space heard best hour better true during hundred five remember step early hold west ground interest reach fast verb sing listen six table travel less morning ten simple several vowel toward war lay against pattern slow center love person money serve appear road map rain rule govern pull cold notice voice unit power town fine certain fly fall lead cry dark machine note wait plan figure star box noun field rest correct able pound done beauty drive stood contain front teach week final gave green oh quick develop ocean warm free minute strong special mind behind clear tail produce fact street inch multiply nothing course stay wheel full force blue object decide surface deep moon island foot system busy test record boat common gold possible plane stead dry wonder laugh thousand ago ran check game shape equate hot miss brought heat snow tire bring yes distant fill east paint language among grand ball yet wave drop heart am present heavy dance engine position arm wide sail material size vary settle speak weight general ice matter circle pair include divide syllable felt perhaps pick sudden count square reason length represent art subject region energy hunt probable bed brother egg ride cell believe fraction forest sit race window store summer train sleep prove lone leg exercise wall catch mount wish sky board joy winter sat written wild instrument kept glass grass cow job edge sign visit past soft fun bright gas weather month million bear finish happy hope flower clothe strange gone jump baby eight village meet root buy raise solve metal whether push seven paragraph third shall held hair describe cook floor either result burn hill safe cat century consider type law bit coast copy phrase silent tall sand soil roll temperature finger industry value fight lie beat excite natural view sense ear else quite broke case middle kill son lake moment scale loud spring observe child straight consonant nation dictionary milk speed method organ pay age section dress cloud surprise quiet stone tiny climb cool design poor lot experiment bottom key iron single stick flat twenty skin smile crease hole trade melody trip office receive row mouth exact symbol die least trouble shout except wrote seed tone join suggest clean break lady yard rise bad blow oil blood touch grew cent mix team wire cost lost brown wear garden equal sent choose fell fit flow fair bank collect save control decimal gentle captain practice separate difficult doctor please protect noon whose locate ring character insect caught period indicate radio spoke atom human history effect electric expect crop modern element hit student corner party supply bone rail imagine provide agree thus capital wont chair danger fruit rich thick soldier process operate guess necessary sharp wing create neighbor wash bat rather crowd corn compare poem string bell depend meat rub tube famous dollar stream fear sight thin triangle planet hurry chief colony clock mine tie enter major fresh search send yellow gun allow print dead spot desert suit current lift rose continue block chart hat sell success company subtract event particular deal swim term opposite wife shoe shoulder spread arrange camp invent cotton born determine quart nine truck noise level chance gather shop stretch throw shine property column molecule select wrong gray repeat require broad prepare salt nose plural anger claim continent oxygen sugar death pretty skill season solution magnet silver thank branch match suffix especially fig afraid huge sister steel discuss forward similar guide experience score apple bought led pitch coat mass card band rope slip win dream evening condition feed tool total basic smell valley nor double seat arrive master track parent shore division sheet substance favor connect post spend chord fat glad original share station dad bread charge proper bar offer segment duck instant market degree populate dear enemy reply drink occur support speech nature range steam motion path liquid log meant quotient teeth shell neck";
    words = words.split(' ');
    words = shuffle(words);
    return words[0] + words[2] + words[5];
}

function qcURL () {
    var _calkey = localStorage.getItem('_calkey');
    if (!window.displayKey) window.displayKey = trigraph();
    return 'https://quik.chat/' + window.displayKey + '?cover=1&noshow=1&meetingroom=' + _calkey;
}

localStorage.setItem('_currentMeetingURL', '');
localStorage.setItem('_currentTabletURL', '');

// Process Meetings & Update screen
function processMeetings () {
    if (!window.events || !Array.isArray(window.events)) {
        goToTVURL(qcURL());
        return;
    }

    var D = + new Date(), currentEvent = null;

    // Find first current meeting
    if (window.events) events.forEach(function (event) {
        var eT = + (new Date(event.end.dateTime + 'Z'));
        var sT = + (new Date(event.start.dateTime + 'Z'));

        if (!currentEvent && D > sT && D < eT) {
            // This is our first current event
            currentEvent = event;
        }
    });

    if (!currentEvent || !currentEvent.body) {
        goToTVURL(qcURL());
        return;
    }

    // var subject = currentEvent.subject;
    var bodyContent = currentEvent.body.content;
    if (!bodyContent) return;

    var meetingUrl = null, meetingType = null;

    // Iterate through all links in bodyContent
    // bodyContent.split(/((http|https)\:\/\/[a-zA-Z0-9\-\.]+\.[a-zA-Z]{2,3}(\/[\S][^"<>\s]*)?)/g).forEach(link => {
    //    if (link && link.includes('http') && link.length > 5) console.log(link);
    // });

    // Find Microsoft Teams meeting URL
    if (bodyContent.includes('https://teams.microsoft.com')) {
        var parts = bodyContent.split('https://teams.microsoft.com');
        meetingUrl = 'https://teams.microsoft.com' + parts[1].split('"')[0];
        meetingType = 'teams';
    }

    // Find teams onlineMeeting (New)
    if (currentEvent.onlineMeeting && currentEvent.onlineMeeting.joinUrl && currentEvent.onlineMeeting.joinUrl.includes('https://teams.microsoft.com')) {
        meetingUrl = currentEvent.onlineMeeting.joinUrl;
        meetingType = 'teams';
    }

    // Find Hangouts meeting URL
    if (bodyContent.includes('https://hangouts.google.com')) {
        var parts = bodyContent.split('https://hangouts.google.com');
        meetingUrl = 'https://hangouts.google.com' + parts[1].split('"')[0];
        meetingType = 'hangouts';
    }

    // Find Google Meet meeting URL
    if (bodyContent.includes('https://meet.google.com')) {
        var parts = bodyContent.split('https://meet.google.com');
        meetingUrl = 'https://meet.google.com' + parts[1].split('"')[0];
        meetingType = 'google-meet';
    }

    // Find Daily.co meeting URL
    if (bodyContent.includes('daily.co/')) {
        var parts = bodyContent.split(/(https:\/\/\w+\.daily\.co\/\w+)/g);

        parts.forEach(part => {
            if (part.includes('daily.co/') && part.includes('https://')) meetingUrl = part;
            meetingType = 'daily';
        });
    }

    // Find Bluejeans meeting URL
    if (bodyContent.includes('https://bluejeans.com/')) {
        var parts = bodyContent.split('https://bluejeans.com');
        meetingUrl = 'https://bluejeans.com' + parts[1].split('"')[0];
        meetingType = 'bluejeans';
    }

    // Find Appear.in meeting URL
    if (bodyContent.includes('https://appear.in/')) {
        var parts = bodyContent.split('https://appear.in');
        meetingUrl = 'https://appear.in' + parts[1].split('"')[0];
        meetingType = 'bluejeans';
    }

    // Find Zoom meeting URL
    if (bodyContent.includes('https://zoom.us/')) {
        var parts = bodyContent.split('https://zoom.us');
        meetingUrl = 'https://zoom.us' + parts[1].split('"')[0];
        meetingType = 'zoom';
    }

    // Find Webex meeting URL
    if (bodyContent.includes('webex.com/')) {
        var parts = bodyContent.split(/(https:\/\/\w+\.webex\.com\/\w+)/g);

        parts.forEach(part => {
            if (part.includes('webex.com/') && part.includes('https://')) meetingUrl = part;
            meetingType = 'webex';
        });
    }

    // Find GoToMeeting meeting URL
    if (bodyContent.includes('https://global.gotomeeting.com/')) {
        var parts = bodyContent.split('https://global.gotomeeting.com');
        meetingUrl = 'https://global.gotomeeting.com' + parts[1].split('"')[0];
        meetingType = 'gotomeeting';
    }


    // Navigate to our new URL if we're not already there
    if (meetingUrl) goToTVURL(meetingUrl);
}

// Get Meetings every 5m (REST)
function getMeetings () {
    var _calkey = localStorage.getItem('_calkey');

    if (_calkey) $.getJSON('https://app.mr365.co/legacyMeetings/' + _calkey + '?includeBody=1', function (data) {
        if (data.events) window.events = data.events;
        processMeetings();
    });
}

chrome.alarms.create('processMeetings', {
    delayInMinutes: 1,
    periodInMinutes: 1
});

chrome.alarms.create('getMeetings', {
    delayInMinutes: 3,
    periodInMinutes: 3
});

chrome.alarms.onAlarm.addListener(function (alarm) {
    if (alarm.name == 'processMeetings') processMeetings();
    if (alarm.name == 'getMeetings') getMeetings();
});

// setInterval(processMeetings, 15 * 1000);
// setInterval(getMeetings, 270 * 1000);
goToTVURL(qcURL());
getMeetings();
