var muted = false, volume = 1, camera = true, mic = true;

var i18n = {
    other: 'other',
    others: 'others'
};

function tp(n) {
    return n > 9 ? "" + n : "0" + n;
}

function month(i) {
    return [
        "January",
        "February",
        "March",
        "April",
        "May",
        "June",
        "July",
        "August",
        "September",
        "October",
        "November",
        "December"
    ][i];
}

function dayOfWeek(i) {
    return [
        "Sunday",
        "Monday",
        "Tuesday",
        "Wednesday",
        "Thursday",
        "Friday",
        "Saturday"
    ][i];
}

function hrs(n) {
    return n === 0 || n === 24 ? 12 : n > 12 ? n - 12 : n;
}

function ampm(n) {
    return n >= 12 ? "pm" : "am";
}

function others(n) {
    if (!i18n || !i18n.other || !i18n.others) return '';

    return n == 1 ? i18n.other : i18n.others;
}

function currentTimeString() {
    let D = new Date();
    return hrs(D.getHours()) + ":" + tp(D.getMinutes()) + ampm(D.getHours());
}

function currentDateString() {
    let D = new Date();
    return (
        dayOfWeek(D.getDay()) +
        ", " +
        month(D.getMonth()) +
        " " +
        D.getDate() +
        ", " +
        D.getFullYear()
    );
}

function displayConfigByKeyUrl (key) {
    var CONFBUCKETURL = 'https://mr365confs.b-cdn.net';
    return CONFBUCKETURL + '/key-' + key + '.json';
}

function getDisplayConfigByKey (key, cb) {
    $.get(displayConfigByKeyUrl(key), function (data) {
        if (window.displayConfig && typeof displayConfig === 'object') window.displayConfig = Object.assign(window.displayConfig, data);
        else window.displayConfig = data;

        if (cb && typeof cb === 'function') cb(window.displayConfig);
    });
}

function setLocalStorage(key, value) {
    if (!key || !value) return;

    try { localStorage.setItem(key, value); } catch (e) {
        console.log(e);
    }
}

function removeLocalStorage(key) {
    try {
        localStorage.removeItem(key);
        return true;
    } catch (e) {
        return false;
    }
}

function getLocalStorage(key) {
    var result = '';

    try { result = localStorage.getItem(key); } catch (e) {
        console.log(e);
        return result;
    }

    return result;
}

function getSearchParam(key) {
    var val = false;
    if (location.search.indexOf(key) !== -1) val = location.search.split(key + '=')[1];
    if (val && val.indexOf('&') !== -1) val = val.split('&')[0];
    return val;
}

function setMuted (state) {

    if (state) setVolume(0);
    else setVolume(volume);

    //chrome.tabs.query({ }, function (tabs) {
    //    tabs.forEach(function (tab) {
    //        if (tab && tab.id) {
    //            chrome.tabs.update(tab.id, { muted: state });
    //        }
    //    });
    //});
}

function setVolume (volume) {
    if (volume) muted = false;

    chrome.tabs.query({ audible: true }, function (tabs) {
        tabs.forEach(function (tab) {
            if (tab && tab.id) {
                console.log('audiotab:', tab.id);
                chrome.runtime.sendMessage(chrome.runtime.id, { action: 'popup-volume-change', tabId: tab.id, sliderValue: (volume * 100) });
            }
        });
    });
}

function volumeUp () {
    if (volume < 2.5) volume += 0.1;
    setVolume(volume);
}

function volumeDown () {
    if (volume > 0) volume -= 0.1;
    setVolume(volume);
}

// Send a command to the child window
function sendCommand (command) {
    chrome.runtime.sendMessage(chrome.runtime.id, { action: 'remote-command', command: command });
}

function updateCamera () {
    if (camera) sendCommand('video-on');
    else sendCommand('video-off');
}
function updateMic () {
    if (mic) sendCommand('mic-on');
    else sendCommand('mic-off');
}

// Process Meetings & Update screen
function processMeetings () {

    function roomAvailable () {
        $('#title').html('Room Available');
        $('.buttons').addClass('hidden');
        $('#times, #desc').html('');
        window.callEnded = false;
    }

    function renderEvent (event, isCurrentMeeting) {
        var sT = event.start.dateTime + 'Z', eT = event.end.dateTime + 'Z', subject = event.subject, organizer = event.organizer.emailAddress.name;
        var sD = new Date(sT), eD = new Date(eT), D = new Date(), attendeeCount = event.attendees.length - 1;

        var timeString = hrs(sD.getHours()) + ':' + tp(sD.getMinutes()) + ampm(sD.getHours()) + ' – ' + hrs(eD.getHours()) + ':' + tp(eD.getMinutes()) + ampm(eD.getHours());

        var attendeeString = organizer;
        if (attendeeCount > 0) attendeeString += ' and ' + attendeeCount + ' ' + others(attendeeCount);

        $('#desc').html(attendeeString);
        $('#times').html(timeString);
        $('#title').html(subject);

        if (isCurrentMeeting) {

            if (subject != window._lastMeetingSubject) window.callEnded = false;
            window._lastMeetingSubject = subject;

            var bodyContent = currentEvent.body.content;

            if (!bodyContent) {
                window.callEnded = false;
                return;
            }

            // Find teams meeting URL
            var parts = bodyContent.split('https://teams.microsoft.com');
            var teamsMeetingURL = 'https://teams.microsoft.com' + parts[1].split('"')[0];

            if (teamsMeetingURL && !window.callEnded) {
                // Show Buttons
                $('.in-call').removeClass('hidden');
                $('.call-ended').addClass('hidden');
            } else {
                // Call ended screen
                $('.call-ended').removeClass('hidden');
                $('.in-call').addClass('hidden');
            }
        } else {
            // Hide all controls if not the current meeting
            $('.in-call, .call-ended').addClass('hidden');
        }
    }

    if (!window.events || !Array.isArray(window.events)) {
        roomAvailable();
        return;
    }

    var D = + new Date(), currentEvent = null, nextEvent = null;

    // Find first current meeting
    if (window.events) events.forEach(function (event) {
        var eT = + (new Date(event.end.dateTime + 'Z'));
        var sT = + (new Date(event.start.dateTime + 'Z'));

        if (!currentEvent && D > sT && D < eT) {
            // This is our first current event
            currentEvent = event;
        }

        if (!nextEvent && D < sT && D < eT) {
            // This is our next event
            nextEvent = event;
        }
    });

    if (currentEvent) {
        $('.status').text('CURRENT MEETING');
        renderEvent(currentEvent, true);
    } else if (nextEvent) {
        $('.status').text('UPCOMING MEETING');
        renderEvent(nextEvent);
    } else {
        roomAvailable();
    }

}

function updateCalendarStatus () {
    var _calkey = getLocalStorage('_calkey');
    if (!_calkey) return;

    if (!window.displayConfig) getDisplayConfigByKey(_calkey, function () {
        if (displayConfig.name) $('#name').text(displayConfig.name);
    });

    $.getJSON('https://app.mr365.co/legacyMeetings/' + _calkey + '?includeBody=1', function (data) {
        if (data.events) window.events = data.events;
        processMeetings();
    });
}

$(document).ready(function() {
    $("button").on("mousedown", function() {
        setTimeout(() => { $(this).removeClass("pressing"); }, 250);
        $(this).addClass("pressing");
    });

    $(".camera").click(function() {
        camera = !camera;

        updateCamera();

        setTimeout(() => {
            $(this).toggleClass("disabled");
            $(this).find("i").toggleClass("slashed");
        }, 150);
    });

    $(".mic").click(function() {
        mic = !mic;

        updateMic();

        setTimeout(() => {
            $(this).toggleClass("disabled");
            $(this).find("i").toggleClass("fa-microphone fa-microphone-slash");
        }, 150);
    });

    $(".vol-off").click(function() {
        muted = !muted;

        setMuted(muted);

        setTimeout(() => {
            $('.vol-up, .vol-down').toggleClass("disabled");
        }, 150);
    });

    $(".end-call").click(function() {
        sendCommand('end-call');
        window.callEnded = true;

        $('.in-call').addClass('hidden');
        $('.call-ended').removeClass('hidden');
    });

    $(".join-call").click(function() {
        sendCommand('rejoin-call');
        window.callEnded = false;

        $('.in-call').removeClass('hidden');
        $('.call-ended').addClass('hidden');

        setTimeout(function () {
            updateCamera();
            updateMic();
        }, 4000);
    });

    function volFx () { $('.vol-up, .vol-down').removeClass("disabled"); }

    $(".vol-up").click(function() {
        setTimeout(volFx, 150);
        volumeUp();
    });

    $(".vol-down").click(function() {
        setTimeout(volFx, 150);
        volumeDown();
    });

    function updateClock() {
        $(".clock h1").text(currentDateString());
        $(".clock h2").text(currentTimeString());
    }

    // setInterval(updateMic, 9 * 1000);
    // setInterval(updateCamera, 7 * 1000);
    setInterval(updateCalendarStatus, 3 * 60 * 1000);
    setInterval(processMeetings, 30 * 1000);
    setInterval(updateClock, 20 * 1000);
    updateCalendarStatus();
    updateClock();
});
